-- DeleteCheapestItem.lua
-- Version 4.4
-- https://www.curseforge.com/wow/addons/delete-cheapest-item

--global vars
DCI_InvErrWaitForWindow = false
DCI_QuestRewardSelected = 1
DCI_LootIndexUserClick = nil

--func to dump table to string, for debugging
local function dump(o)
   if type(o) == 'table' then
      local s = '{ '
      for k,v in pairs(o) do
         if type(k) ~= 'number' then k = '"'..k..'"' end
         s = s .. '['..k..'] = ' .. dump(v) .. ','
      end
      return s .. '} '
   else
      return tostring(o)
   end
end

--function for debug output
local function DCIPrint(message, outputType)
    --param: outputType = 0 forced, 1 info, 2 warning, 3 error
    if outputType == nil then outputType = 1 end

    --quit if below threshold setting
    if outputType > 0 and (DCI_DebugOutput == nil or outputType < DCI_DebugOutput) then
        return false
    end

    message = dump(message)

    local colorCode
    if outputType == 3 then colorCode = "|cffff1111"
    elseif outputType == 2 then colorCode = "|cffffff00"
    elseif outputType == 1 then colorCode = "|cffaaaaaa"
    else colorCode = "|cffffffff"
    end

    DEFAULT_CHAT_FRAME:AddMessage("|cff7f7fff" .. "[DeleteCheapestItem] " .. colorCode .. message)
end

--function to format and truncate money string
local function GetMoneyStringTruncated(coppers, truncateCoppersAfter, truncateSilversAfter)
    --param: truncateCoppersAfter - after this amount, round down to 100s
    if truncateCoppersAfter == nil then truncateCoppersAfter = 10000 end
    --param: truncateSilverAfter - after this amount, round down to 10000s
    if truncateSilversAfter == nil then truncateSilversAfter = 10000000 end

    local truncatedCoppers    
    if truncateSilversAfter and coppers > truncateSilversAfter then
        truncatedCoppers = math.floor(coppers / 10000) * 10000
    elseif truncateCoppersAfter and coppers > truncateCoppersAfter then
        truncatedCoppers = math.floor(coppers / 100) * 100
    else
        truncatedCoppers = coppers
    end

    return GetMoneyString(truncatedCoppers)
end

--function to check if auction prices are enabled and available
local function UseAuctionPrices(forceEnabled)
    --param: forceEnabled (boolean) - if true, overrides user setting DCI_UseAuctionPrices
    if forceEnabled == nil then forceEnabled = false end

    --only check APIs if we haven't already
    if not DCI_AuctionSource then
        --check for all supported APIs
        if (Auctionator and Auctionator.API and Auctionator.API.v1 and Auctionator.API.v1.GetAuctionPriceByItemID) then
            DCI_AuctionSource = "Auctionator"
        elseif (AucAdvanced and AucAdvanced.API and AucAdvanced.API.GetMarketValue and AucAdvanced.GetFaction) then
            DCI_AuctionSource = "Auctioneer"
        elseif (AuctionLite and AuctionLite.GetAuctionValue) then
            DCI_AuctionSource = "AuctionLite"
        elseif (vendor and vendor.Statistic) then
            DCI_AuctionSource = "AuctionMaster"
        elseif (TSM_API and TSM_API.GetCustomPriceValue) then
            DCI_AuctionSource = "TSM"
        else
            --if no source API was found return false
            return false
        end
    end

    --if we made it this far (api exists), then go by the global user setting
    return (DCI_UseAuctionPrices or forceEnabled) 
end

--function to save an item to the ignore list
local function SaveIgnoredItem(itemID, itemName)
    --parameter: itemName - default to itemID
    if itemName == nil then itemName = itemID end

    local IgnoredItems_copy = DCI_IgnoredItems
    table.insert(IgnoredItems_copy, itemID)
    DCI_IgnoredItems = IgnoredItems_copy

    DCIPrint("Ignoring item: "..itemName, 0)
end

--function to check if an itemID is in the saved variable ignored list
local function IsItemIgnored(itemID)

    --check if ignored list exists
    if not DCI_IgnoredItems then
        return false
    end

    -- Check if the itemID is in the list
    for _, storedItemID in ipairs(DCI_IgnoredItems) do
        if storedItemID == itemID then
            return true  -- ItemID found in the list
        end
    end

    return false  -- ItemID not found in the list
end

--function to get auction price by itemID
local function GetAuctionPriceByItemID(itemID)
    --quit if global auction prices disabled
    if UseAuctionPrices() == false then return nil end

    local finalAuctionPrice = nil
    local _,itemLink = GetItemInfo(itemID);

    --try to get price from different auction addons
    if (Auctionator and Auctionator.API and Auctionator.API.v1 and Auctionator.API.v1.GetAuctionPriceByItemID) then
        --Auctionator
        finalAuctionPrice = Auctionator.API.v1.GetAuctionPriceByItemID("DeleteCheapestItem", itemID)

    elseif (AucAdvanced and AucAdvanced.API and AucAdvanced.API.GetMarketValue and AucAdvanced.GetFaction) then
        --Auctioneer
        finalAuctionPrice, _ = AucAdvanced.API.GetMarketValue(itemLink, AucAdvanced.GetFaction())

    elseif (AuctionLite and AuctionLite.GetAuctionValue) then
        --AuctionLite
        finalAuctionPrice = AuctionLite:GetAuctionValue(itemID)

    elseif (vendor and vendor.Statistic) then
        --AuctionMaster
        _, finalAuctionPrice, _, _ = vendor.Statistic:GetCurrentAuctionInfo(itemLink, false)

    elseif (TSM_API and TSM_API.GetCustomPriceValue) then
        --TSM
        local itemString = TSM_API.ToItemString(itemLink);
        finalAuctionPrice = TSM_API.GetCustomPriceValue("Dbminbuyout", itemString);

    else
        --no auction API found
        return nil
    end

    --subtract 5% auction cut
    if DCI_SubtractAuctionCut and finalAuctionPrice then
        finalAuctionPrice = finalAuctionPrice * 0.95
    end

    return finalAuctionPrice
end

--function to get the items from the open quest reward window
local function GetQuestRewardWindowItems()
    DCIPrint("Getting quest reward items")
    local lootItems = {}

    --loop optional reward choices
    for i = 1, GetNumQuestChoices() do
        lootItems[i] = {}
        _,_,lootItems[i].quantity,_,_ = GetQuestItemInfo("choice", i)
        local itemLink = GetQuestItemLink("choice", i)
        if itemLink then
            lootItems[i].itemID = tonumber(string.match(itemLink, "item:(%d+):"))
        end
    end

    --loop forced rewards
    for i = 1, GetNumQuestRewards() do
        lootItems[GetNumQuestChoices() + i] = {}
        _,_,lootItems[GetNumQuestChoices() + i].quantity,_,_ = GetQuestItemInfo("reward", i)
        local itemLink = GetQuestItemLink("reward", i)
        if itemLink then
            lootItems[GetNumQuestChoices() + i].itemID = tonumber(string.match(itemLink, "item:(%d+):"))
        end
    end

    --gather info for all items
    DCIPrint("Getting info for quest items:")
    DCIPrint(lootItems)
    for i = 1, #lootItems do
        local itemName,itemLink,itemRarity,_,_,_,_,_,_,_,sellPrice = GetItemInfo(lootItems[i].itemID)
    
        lootItems[i].isQuestItem = false
        lootItems[i].itemName = itemName
        lootItems[i].itemLink = itemLink
        lootItems[i].itemRarity = itemRarity

        lootItems[i].sellPrice = sellPrice * lootItems[i].quantity
        lootItems[i].bestPrice = lootItems[i].sellPrice
        lootItems[i].bestPriceType = 1

        --if using auction prices [and not grey] and comparing by auction or highest price, get auction prices
        if UseAuctionPrices() and DCI_CompareByPrice > 1 and lootItems[i].itemID ~= nil and lootItems[i].itemRarity ~= 0 then
            local auctionPriceResult = GetAuctionPriceByItemID(lootItems[i].itemID)
            if auctionPriceResult == nil then
                lootItems[i].auctionPrice = nil
            else
                lootItems[i].auctionPrice = auctionPriceResult * lootItems[i].quantity
                if lootItems[i].auctionPrice > lootItems[i].sellPrice then
                    lootItems[i].bestPrice = lootItems[i].auctionPrice 
                    lootItems[i].bestPriceType = 2
                end
            end
        end
    end

    return lootItems
end

--function to get the items from the open loot window, sorted by value
local function GetSortedLootWindowItems()
    DCIPrint("Getting sorted loot items")
    local lootSlotValue = {}
    local lootSlotNum = 0
    for i = 1, GetNumLootItems() do
        if LootSlotHasItem(i) and GetLootSlotLink(i) then
            lootSlotNum = lootSlotNum + 1
            lootSlotValue[lootSlotNum] = {}
            lootSlotValue[lootSlotNum].lootIndex = i
            lootSlotValue[lootSlotNum].sellPrice = 0

            local itemName,itemLink,itemRarity,_,_,_,_,_,_,_,sellPrice = GetItemInfo(GetLootSlotLink(i))
            local _,_,lootQuantity,_,_,_,isQuestItem = GetLootSlotInfo(i)
            if lootQuantity == nil then lootQuantity = 1 end
            if isQuestItem == nil then isQuestItem = false end

            lootSlotValue[lootSlotNum].isQuestItem = isQuestItem
            lootSlotValue[lootSlotNum].itemName = itemName
            lootSlotValue[lootSlotNum].itemLink = itemLink
            lootSlotValue[lootSlotNum].itemID = tonumber(string.match(itemLink, "item:(%d+):"))
            lootSlotValue[lootSlotNum].itemRarity = itemRarity
            lootSlotValue[lootSlotNum].quantity = lootQuantity

            if GetLootSlotType(i) == 1 and not isQuestItem then
                lootSlotValue[lootSlotNum].sellPrice = sellPrice * lootQuantity
                lootSlotValue[lootSlotNum].bestPrice = lootSlotValue[lootSlotNum].sellPrice
                lootSlotValue[lootSlotNum].bestPriceType = 1

                --if using auction prices [and not grey] and comparing by auction or highest price, get auction prices 
                if UseAuctionPrices() and DCI_CompareByPrice > 1 and lootSlotValue[lootSlotNum].itemID ~= nil and lootSlotValue[lootSlotNum].itemRarity ~= 0 then
                    local auctionPriceResult = GetAuctionPriceByItemID(lootSlotValue[lootSlotNum].itemID)
                    if auctionPriceResult == nil then
                        lootSlotValue[lootSlotNum].auctionPrice = nil
                    else
                        lootSlotValue[lootSlotNum].auctionPrice = auctionPriceResult * lootQuantity
                        if lootSlotValue[lootSlotNum].auctionPrice > lootSlotValue[lootSlotNum].sellPrice then
                            lootSlotValue[lootSlotNum].bestPrice = lootSlotValue[lootSlotNum].auctionPrice 
                            lootSlotValue[lootSlotNum].bestPriceType = 2
                        end
                    end
                end
            end
        end
    end

    DCIPrint(lootSlotValue)

    table.sort(lootSlotValue, function(item1, item2) 
        local val1,val2

        if (item1 == nil or item2 == nil) then
            return false
        elseif item1.isQuestItem then
            --sort quest items to the top
            return true
        elseif item2.isQuestItem then
            return false
        elseif DCI_CompareByPrice == 3 and UseAuctionPrices() then
            --comparison option = bestprice: try to use bestprice, otherwise sellprice
            if item1.bestPrice then val1 = item1.bestPrice
            elseif item1.sellPrice then val1 = item1.sellPrice
            else return false
            end
            if item2.bestPrice then val2 = item2.bestPrice
            elseif item2.sellPrice then val2 = item2.sellPrice
            else return false
            end
        elseif DCI_CompareByPrice == 2 and UseAuctionPrices() then
            --comparison option = auctionprice: try to use auctionprice, otherwise sellprice
            if item1.auctionPrice then val1 = item1.auctionPrice
            elseif item1.sellPrice then val1 = item1.sellPrice
            else return false
            end
            if item2.auctionPrice then val2 = item2.auctionPrice
            elseif item2.sellPrice then val2 = item2.sellPrice
            else return false
            end
        else
            --otherwise just use sellprice
            if item1.sellPrice then val1 = item1.sellPrice
            else return false
            end
            if item2.sellPrice then val2 = item2.sellPrice
            else return false
            end
        end

        return val1 > val2
    end)

    return lootSlotValue
end

--function to loot items in the open loot window, used after making space in bags
local function TakeItemsFromLootWindow(forcedFirstLootIndex)
    DCIPrint("Looting next item in window")
    --get quest items & cash first
    local lootSlotType = {}
    for i = 1, GetNumLootItems() do
        if LootSlotHasItem(i) then
            local _,_,_,_,_,_,isQuestItem = GetLootSlotInfo(i)
            lootSlotType[i] = GetLootSlotType(i)
            if isQuestItem or lootSlotType[i] > 1 then
                LootSlot(i)
            end
        end
    end

    --forcedFirstLootIndex
    if forcedFirstLootIndex then
        DCIPrint("Using forced loot index")
        LootSlot(forcedFirstLootIndex)
    end

    --loot in order of value
    local lootSlotValue = GetSortedLootWindowItems()
    for i,nextLootItem in pairs(lootSlotValue) do
        if forcedFirstLootIndex == nil or nextLootItem.lootIndex ~= forcedFirstLootIndex then --skip the forced one we aready did
            LootSlot(nextLootItem.lootIndex)
        end
    end
end

--function to get all items in bags and sort by price
local function GetSortedBagItems()
    DCIPrint("Getting sorted bag items")
    --Create item array
	local bagItems = {}
    local bagItemCount = 0

    -- Iterate through all bags (bag indices 0 to 4)
    for bag = 0, NUM_BAG_SLOTS do
        -- Get the number of slots in the bag
        local numSlots = C_Container.GetContainerNumSlots(bag)

        -- Iterate through all slots in the bag
        for slot = 1, numSlots do
       	        	
            -- Get information about the item in the slot
            -- icon, itemCount, locked, quality, readable, lootable, itemLink, isFiltered, noValue, itemID, isBound
            local newBagItem = C_Container.GetContainerItemInfo(bag, slot)

            if newBagItem ~= nil and not newBagItem.hasNoValue then
                --check if quest item
                ItemQuestInfo = C_Container.GetContainerItemQuestInfo(bag, slot)
                ItemIsSoulbound = C_Item.IsBound(ItemLocation:CreateFromBagAndSlot(bag, slot)) or false

                --skip ignored items or quest items 
                --or soulbound items based on global DCI_HideSoulbound
                --or quality is above threshold
                if (not IsItemIgnored(newBagItem.itemID) and not ItemQuestInfo.isQuestItem
                    and (DCI_HideSoulbound == false or ItemIsSoulbound == false)
                    and newBagItem.quality <= DCI_MaxQuality) then
    	            -- Append info
    	            newBagItem.bag = bag
    	            newBagItem.slot = slot
                    local _,_,itemQuality,_,_,_,_,_,_,_,itemPrice = GetItemInfo(newBagItem.itemID)
    	            newBagItem.vendorPricePer = itemPrice
    	            newBagItem.vendorPriceTotal = newBagItem.vendorPricePer * newBagItem.stackCount
                    newBagItem.bestPricePer = newBagItem.vendorPricePer

                    --get auction prices if flag to use auction is set, and the item is not soulbound
                    if UseAuctionPrices() and not ItemIsSoulbound then
                        --also check if grey based on global setting DCI_AuctionPriceHidePoorQuality
                        if DCI_AuctionPriceHidePoorQuality == false or itemQuality > 0 then
                            newBagItem.auctionPricePer = GetAuctionPriceByItemID(newBagItem.itemID)
                            if not newBagItem.auctionPricePer then
                                newBagItem.auctionPricePer = 0
                            end
                            newBagItem.auctionPriceTotal = newBagItem.auctionPricePer * newBagItem.stackCount
                            newBagItem.bestPricePer = max(newBagItem.vendorPricePer, newBagItem.auctionPricePer)
                        end
                    end

                    newBagItem.bestPriceTotal = newBagItem.bestPricePer * newBagItem.stackCount

    	            -- add it to array
    				bagItemCount = bagItemCount + 1
    	            bagItems[bagItemCount] = newBagItem
                end
		    end
        end
    end

    table.sort(bagItems, function(item1, item2) 
        local val1,val2

        if (item1 == nil or item2 == nil) then
            return false
        elseif DCI_CompareByPrice == 3 and UseAuctionPrices() then
            --comparison option = bestprice: try to use bestprice, otherwise sellprice
            if item1.bestPriceTotal then val1 = item1.bestPriceTotal
            elseif item1.vendorPriceTotal then val1 = item1.vendorPriceTotal
            else return false
            end
            if item2.bestPriceTotal then val2 = item2.bestPriceTotal
            elseif item2.vendorPriceTotal then val2 = item2.vendorPriceTotal
            else return false
            end
        elseif DCI_CompareByPrice == 2 and UseAuctionPrices() then
            --comparison option = auctionprice: try to use auctionprice, otherwise sellprice
            if item1.auctionPriceTotal then val1 = item1.auctionPriceTotal
            elseif item1.vendorPriceTotal then val1 = item1.vendorPriceTotal
            else return false
            end
            if item2.auctionPriceTotal then val2 = item2.auctionPriceTotal
            elseif item2.vendorPriceTotal then val2 = item2.vendorPriceTotal
            else return false
            end
        else
            --otherwise just use sellprice
            if item1.vendorPriceTotal then val1 = item1.vendorPriceTotal
            else return false
            end
            if item2.vendorPriceTotal then val2 = item2.vendorPriceTotal
            else return false
            end
        end

        return val1 < val2
    end)

    return bagItems --{unpack(bagItems, 1, 3)}
end

--function to count free bag slots
local function CountFreeBagSlots()
    local freeSlots = 0

    -- Iterate through all bags (bag indices 0 to 4)
    for bag = 0, NUM_BAG_SLOTS do
        -- Get the number of slots in the bag
        local numSlots = C_Container.GetContainerNumSlots(bag)

        -- Iterate through all slots in the bag
        for slot = 1, numSlots do
            -- Check if the slot is empty
            if nil == C_Container.GetContainerItemInfo(bag, slot) then
                freeSlots = freeSlots + 1
            end
        end
    end

    DCIPrint("Free bag slots: " .. freeSlots)

    return freeSlots
end

--function to delete item, with optional confirmation window
local function ConfirmAndDeleteBagItem(bag, slot)
    --local function to actually delete, serious this time
    local function DeleteBagItemForReal(bag, slot, itemText)
        --parameter: itemText - default to "Unknown"
        if itemText == nil then itemText = "Unknown" end

        C_Container.PickupContainerItem(bag, slot)
        DeleteCursorItem()
        DCIPrint("Deleting item: "..itemText, 0)
    end

    --format item name for display
    local itemInfo = C_Container.GetContainerItemInfo(bag, slot)
    local itemText = nil
    if itemInfo.hyperlink then
        if itemInfo.stackCount and itemInfo.stackCount > 1 then itemText = itemInfo.stackCount .. "x" end
        itemText = (itemText or "") .. itemInfo.hyperlink
    end

    --check if we should show confirmation based on DCI_ConfirmMinQuality
    if DCI_ConfirmMinQuality and itemInfo.quality >= DCI_ConfirmMinQuality then
        --confirmation is required before we delete

        StaticPopupDialogs["DELETE_ITEM_CONFIRMATION"] = {
            text = "Are you sure you want to delete " .. (itemText or "this item").. "?",
            button1 = "Yes",
            button2 = "No",
            OnAccept = function() DeleteBagItemForReal(bag, slot, itemText) end,
            timeout = 0,
            whileDead = true,
            hideOnEscape = true,
            preferredIndex = 3
        }

        StaticPopup_Show("DELETE_ITEM_CONFIRMATION")
    else
        --no confirmation required, delete immediately
        DeleteBagItemForReal(bag, slot, itemText)
    end

end

--function to create and show a frame with cheapest items
local function CreateDCIFrame(items)
    -- Create a frame
    if DCIFrame == nil then 
        DCIFrame = CreateFrame("Frame", "DCIFrame", UIParent, "BasicFrameTemplate")
    end
    DCIFrame:SetSize(280, 240)

    -- if loot frame is open, anchor to it
    DCIFrame:ClearAllPoints()
    if LootFrame:IsShown() then
        DCIFrame:SetPoint("LEFT", LootFrame, "RIGHT", 10, 0) 
    elseif LootFrame.numLootItems then
        --LootFrame is hidden, but has been used and we can anchor to it
        DCIFrame:SetPoint("LEFT", LootFrame, "LEFT", 0, 0)
    else
        DCIFrame:SetPoint("LEFT", 10, 0) 
    end

    --title
    DCI_titleText = DCIFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
    DCI_titleText:SetPoint("TOPLEFT", DCIFrame, "TOPLEFT", 0, 0)
    DCI_titleText:SetPoint("BOTTOMRIGHT", DCIFrame, "TOPRIGHT", -25, -24)
    DCI_titleText:SetText("Delete Cheapest Item")

    --dragable title bar handlers
    DCIFrame:SetMovable(true)
    DCI_titleText:SetScript("OnMouseDown", function(self, button)
        if button == "LeftButton" then
            DCIFrame:StartMoving()
            DCI_WindowMoved = true
        end
    end)

    DCI_titleText:SetScript("OnMouseUp", function(self, button)
        if button == "LeftButton" then
            DCIFrame:StopMovingOrSizing()
        end
    end)

    --info button
    infoButton = CreateFrame("Button", "DCI_InfoButton", DCIFrame, "UIPanelInfoButton")
    infoButton:SetPoint("TOPRIGHT", DCIFrame, "TOPRIGHT", -21, -3)
    infoButton:SetSize(23, 23)

    --info button tooltip
    infoButton:SetScript("OnEnter", function()
        GameTooltip:SetOwner(DCIFrame, "ANCHOR_NONE")
        GameTooltip:SetPoint("TOPLEFT", DCIFrame, "TOPLEFT", -2, -20) 

        GameTooltip:AddLine("Settings (Click to Change)", 1, 1, 1, 1, true)

        GameTooltip:AddLine(" ")

        GameTooltip:AddDoubleLine("Allow window in combat", DCI_AllowInCombat and "YES" or "NO", 1.0, 0.82, 0.0, DCI_AllowInCombat and 0 or 1, DCI_AllowInCombat and 1 or 0, 0)
        GameTooltip:AddDoubleLine("Hide soulbound items", DCI_HideSoulbound and "YES" or "NO", 1.0, 0.82, 0.0, DCI_HideSoulbound and 0 or 1, DCI_HideSoulbound and 1 or 0, 0)

        local qualityOptions = {"Common", "Uncommon", "Rare", "Epic", "Legendary"} qualityOptions[0] = "Poor"
        local confirmQuality = DCI_ConfirmMinQuality and qualityOptions[DCI_ConfirmMinQuality] or "Never"
        local r,g,b,hex = GetItemQualityColor(DCI_MaxQuality)
        GameTooltip:AddDoubleLine("Max item quality to display", qualityOptions[DCI_MaxQuality], 1.0, 0.82, 0.0, r, g, b)

        if DCI_ConfirmMinQuality and DCI_MaxQuality >= DCI_ConfirmMinQuality then
            r,g,b,hex = GetItemQualityColor(DCI_ConfirmMinQuality)
        else
            r,g,b = 1,0,0
            confirmQuality = "Never"
        end
        GameTooltip:AddDoubleLine("Min item quality to confirm delete", confirmQuality, 1.0, 0.82, 0.0, r, g, b)

        GameTooltip:AddLine(" ")

        if DCI_UseAuctionPrices then
            local auctionPriceTooltipText = "Using auction prices from"
        else
            local auctionPriceTooltipText = "Use auction prices"
        end
        GameTooltip:AddDoubleLine("Use auction prices" .. (DCI_UseAuctionPrices and " from" or ""), DCI_UseAuctionPrices and DCI_AuctionSource or "NO", 1.0, 0.82, 0.0, DCI_UseAuctionPrices and 0 or 1, DCI_UseAuctionPrices and 1 or 0, 0)
        if UseAuctionPrices() then
            local compareByOptions = {"Vendor Price", "Auction Price", "Highest Price"}
            GameTooltip:AddDoubleLine("Compare item values by", compareByOptions[DCI_CompareByPrice], 1.0, 0.82, 0.0, 0, 1, 0)
            GameTooltip:AddDoubleLine("Lines to show", DCI_ShowBothPrices and "Both Prices" or "Name + Price", 1.0, 0.82, 0.0, 0, 1, 0)
            if DCI_ShowBothPrices then
                GameTooltip:AddDoubleLine("Grey out auction price if less than vendor price", DCI_AuctionGreyUnderVendor and "YES" or "NO", 1.0, 0.82, 0.0, DCI_AuctionGreyUnderVendor and 0 or 1, DCI_AuctionGreyUnderVendor and 1 or 0, 0)
            end
            GameTooltip:AddDoubleLine("Subtract 5% cut from auction price", DCI_SubtractAuctionCut and "YES" or "NO", 1.0, 0.82, 0.0, DCI_SubtractAuctionCut and 0 or 1, DCI_SubtractAuctionCut and 1 or 0, 0)
            GameTooltip:AddDoubleLine("Ignore auction price for grey quality items", DCI_AuctionPriceHidePoorQuality and "YES" or "NO", 1.0, 0.82, 0.0, DCI_AuctionPriceHidePoorQuality and 0 or 1, DCI_AuctionPriceHidePoorQuality and 1 or 0, 0)
        end

        GameTooltip:AddLine(" ")
        GameTooltip:AddLine("Feedback and suggestions are welcome! :)", 0.3, 0.3, 0.3)

        GameTooltip:Show()
    end)
    infoButton:SetScript("OnLeave", function()
        GameTooltip:Hide()
    end)

    --click event for info button
    infoButton:SetScript("OnMouseUp", function()
        if (SettingsPanel:IsShown() and SettingsPanel.ClosePanelButton) then
            SettingsPanel.ClosePanelButton:Click()
        else
            InterfaceAddOnsList_Update()
            InterfaceOptionsFrame_OpenToCategory("Delete Cheapest Item")
            InterfaceOptionsFrame_OpenToCategory("Delete Cheapest Item")
        end
    end)

    --top text
    DCI_topText1 = DCIFrame:CreateFontString(nil, "OVERLAY", "GameFontDisableSmall")
    DCI_topText1:SetPoint("TOPLEFT", DCIFrame, "TOPLEFT", 8, -31)
    DCI_topText1:SetText("Below are the cheapest items in your bags.")
    
    DCI_topText2 = DCIFrame:CreateFontString(nil, "OVERLAY", "GameFontDisableSmall")
    DCI_topText2:SetPoint("TOPLEFT", DCI_topText1, "BOTTOMLEFT", 0, -5)
    DCI_topText2:SetText("")

    --black background texture
    DCI_blackBackground = DCIFrame:CreateTexture(nil, "BACKGROUND")
    DCI_blackBackground:SetPoint("TOPLEFT", DCIFrame, "TOPLEFT", 0, -62) 
    DCI_blackBackground:SetPoint("BOTTOMRIGHT", DCIFrame, "BOTTOMRIGHT", 0, 0)
    DCI_blackBackground:SetColorTexture(0, 0, 0, 0.7) 

    --loop 4 items
    DCI_itemButton = {}
    DCI_itemTexture = {}
    DCI_itemInfoBackground = {}
    DCI_glowTexture = {}
    DCI_countText = {}
    DCI_deleteButton = {}
    DCI_ignoreButton = {}
    DCI_itemTextTop1 = {}
    DCI_itemTextTop2 = {}
    DCI_itemTextBottom1 = {}
    DCI_itemTextBottom2 = {}
    for itemNum = 1,4 do
        --item icon
        DCI_itemButton[itemNum] = CreateFrame("Button", "DCI_itemButton" .. itemNum, DCIFrame)
        DCI_itemButton[itemNum]:SetSize(38, 38)
        DCI_itemButton[itemNum]:SetPoint("TOPLEFT", DCI_blackBackground, "TOPLEFT", 3, -6 + (itemNum-1) * -42)
        DCI_itemTexture[itemNum] = DCI_itemButton[itemNum]:CreateTexture(nil, "BACKGROUND")
        DCI_itemTexture[itemNum]:SetAllPoints(DCI_itemButton[itemNum])

        --show the quantity
        DCI_countText[itemNum] = DCI_itemButton[itemNum]:CreateFontString(nil, "OVERLAY", "NumberFontNormal")
        DCI_countText[itemNum]:SetPoint("BOTTOMRIGHT", DCI_itemButton[itemNum], "BOTTOMRIGHT", -2, 2)

        --item info box background
        DCI_itemInfoBackground[itemNum] = DCI_itemButton[itemNum]:CreateTexture(nil, "BACKGROUND")
        DCI_itemInfoBackground[itemNum]:SetTexture(136796)
        DCI_itemInfoBackground[itemNum]:SetPoint("TOPLEFT", DCI_itemButton[itemNum], "TOPRIGHT", -10, 12)
        DCI_itemInfoBackground[itemNum]:SetSize(170, 61)
        DCI_itemInfoBackground[itemNum]:SetVertexColor(0.5, 0.5, 0.5, 1)

        --glow texture for highlight (start out hidden)
        DCI_glowTexture[itemNum] = DCI_itemButton[itemNum]:CreateTexture(nil, "OVERLAY")
        DCI_glowTexture[itemNum]:SetTexture("Interface/Buttons/UI-ActionButton-Border")
        DCI_glowTexture[itemNum]:SetBlendMode("ADD")  -- ADD blend mode for a glow effect
        DCI_glowTexture[itemNum]:SetTexCoord(0.28, 0.72, 0.28, 0.72)
        DCI_glowTexture[itemNum]:SetPoint("LEFT", DCIFrame, "LEFT", 3, 0)
        DCI_glowTexture[itemNum]:SetPoint("RIGHT", DCIFrame, "RIGHT", -3, 0)
        DCI_glowTexture[itemNum]:SetPoint("TOP", DCI_itemButton[itemNum], "TOP", 0, 1)
        DCI_glowTexture[itemNum]:SetPoint("BOTTOM", DCI_itemButton[itemNum], "BOTTOM", 0, -1)
        DCI_glowTexture[itemNum]:SetVertexColor(1, 0, 0, 0.5)  -- Adjust the alpha value to control the intensity of the glow
        DCI_glowTexture[itemNum]:Hide()

        --display item name/values
        DCI_itemTextTop1[itemNum] = DCI_itemButton[itemNum]:CreateFontString(nil, "OVERLAY", "GameFontNormal")
        DCI_itemTextTop1[itemNum]:SetPoint("TOPLEFT", DCI_itemInfoBackground[itemNum], "TOPLEFT", 20, -16)
        DCI_itemTextTop2[itemNum] = DCI_itemButton[itemNum]:CreateFontString(nil, "OVERLAY", "GameFontNormal")
        DCI_itemTextTop2[itemNum]:SetPoint("TOPRIGHT", DCI_itemInfoBackground[itemNum], "TOPRIGHT", -20, -16)
        DCI_itemTextTop2[itemNum]:SetText(" ") --to ensure relative positioning works
        DCI_itemTextBottom1[itemNum] = DCI_itemButton[itemNum]:CreateFontString(nil, "OVERLAY", "GameFontNormal")
        DCI_itemTextBottom1[itemNum]:SetPoint("TOPLEFT", DCI_itemTextTop1[itemNum], "BOTTOMLEFT", 0, -5)
        DCI_itemTextBottom2[itemNum] = DCI_itemButton[itemNum]:CreateFontString(nil, "OVERLAY", "GameFontNormal")
        DCI_itemTextBottom2[itemNum]:SetPoint("TOPRIGHT", DCI_itemTextTop2[itemNum], "BOTTOMRIGHT", 0, -5)

        --show delete button
        DCI_deleteButton[itemNum] = CreateFrame("Button","deleteButton",DCI_itemButton[itemNum],"UIPanelButtonTemplate") 
        DCI_deleteButton[itemNum]:SetText("Delete")
        DCI_deleteButton[itemNum]:SetSize(60, 32)  -- Set the size of the texture
        DCI_deleteButton[itemNum]:SetPoint("LEFT", DCI_itemInfoBackground[itemNum], "RIGHT", -10, 0)

        --ignore button. -- UIPanelInfoButtonTemplate UIPanelSquareButton
        DCI_ignoreButton[itemNum] = CreateFrame("Button", "ignoreButton", DCI_itemButton[itemNum], "UIPanelCloseButton")
        DCI_ignoreButton[itemNum]:SetSize(28, 28)
        DCI_ignoreButton[itemNum]:SetPoint("LEFT", DCI_deleteButton[itemNum], "RIGHT", -1, 0)

        -- Set up a tooltip for the ignore
        DCI_ignoreButton[itemNum]:SetScript("OnEnter", function(self)
            GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
            GameTooltip:SetText("Ignore This Item")
            GameTooltip:Show()
        end)
        DCI_ignoreButton[itemNum]:SetScript("OnLeave", function()
            GameTooltip:Hide()
        end)

        DCI_itemButton[itemNum]:Hide()
    end

    --reset clicked item index when window closes
    DCIFrame:SetScript("OnHide", function()
        if DCI_LootIndexUserClick then
            DCIPrint("Resetting click index")
            DCI_LootIndexUserClick = nil
        end
    end)
end

--function to update and display main window
function UpdateDCIFrame(showFrameAfterUpdate)
    --parameter: showFrameAfterUpdate - default true
    if showFrameAfterUpdate == nil then showFrameAfterUpdate = true end
    DCIPrint("UpdateDCIFrame " .. (showFrameAfterUpdate and "true" or "false"))

    --make frame if its missing
    if DCIFrame == nil then
        CreateDCIFrame()
        if not showFrameAfterUpdate then DCIFrame:Hide() end
    end

    --quit if mail frame is open
    if MailFrame:IsShown() then
        DCIPrint("Hiding since mail is open")
        DCIFrame:Hide()
        return false
    end

    --quit if player is in combat based on global DCI_AllowInCombat
    if DCI_AllowInCombat == false and UnitAffectingCombat("player") then
        DCIPrint("Hiding since player is in combat - set wait flag*")
        DCIFrame:Hide()
        --DCI_InvErrWaitForWindow = true
        return false
    end

    --choose where to anchor & get loot items from anchored window
    local lootItems = nil
    local itemIndexToLoot = nil
    if LootFrame:IsShown() then
        --anchor to loot window
        DCIPrint("Anchoring to loot window")
        DCIFrame:ClearAllPoints()
        DCIFrame:SetPoint("LEFT", LootFrame, "RIGHT", 10, 0) 
    
        --get items to be looted
        lootItems = GetSortedLootWindowItems()
        for i,nextLootItem in pairs(lootItems) do
            if nextLootItem and LootSlotHasItem(nextLootItem.lootIndex) then
                itemIndexToLoot = i
                break
            end
        end
    elseif QuestFrame:IsShown() then
        --anchor to quest window
        DCIPrint("Anchoring to quest window")
        DCIFrame:ClearAllPoints()
        DCIFrame:SetPoint("TOPLEFT", QuestFrame, "TOPRIGHT", -20, -20)
    
        --get items to be looted
        lootItems = GetQuestRewardWindowItems()
        DCIPrint("Done getting quest items")
        itemIndexToLoot = DCI_QuestRewardSelected

        --hide with quest frame
        if not QuestFrame.DCIHooked then
            QuestFrame:HookScript("OnHide", function(self)
                DCIPrint("Hiding with quest frame")
                DCIFrame:Hide()
            end)
            QuestFrame.DCIHooked = true
        end
    else
        --anchor to screen
        DCIPrint("Anchor to screen")
        if not DCI_WindowMoved then
            DCIFrame:ClearAllPoints()
            if LootFrame.numLootItems then
                --LootFrame has been used and we can anchor to it even though its not shown
                DCIFrame:SetPoint("LEFT", LootFrame, "LEFT", 0, 0)
            else
                DCIFrame:SetPoint("LEFT", 10, 0) 
            end
        end
    end

    --overwrite item to loot if the user clicks on an item in loot frame
    if DCI_LootIndexUserClick and DCI_LootIndexUserClick <= GetNumLootItems() then
        for i = 1,#lootItems do
            if lootItems[i].lootIndex == DCI_LootIndexUserClick then
                itemIndexToLoot = i
                DCIPrint("Overwriting item to loot with player choice " .. itemIndexToLoot)
                break
            end
        end
    end

    -- if loot frame or quest frame is open then show text with highlight
    if LootFrame:IsShown() or QuestFrame:IsShown() then
        if (itemIndexToLoot) then
            local endOfItemString = ""
            if lootItems[itemIndexToLoot].isQuestItem then
                endOfItemString = " (quest item)"
                DCI_topText2:SetText("Choose an item to delete to make room:")

                highlightValue = nil
            else
                local bestPriceTypeOptions = {"Vendor", "Auction"}
                endOfItemString = "  (" .. bestPriceTypeOptions[lootItems[itemIndexToLoot].bestPriceType] .. ": " .. GetMoneyStringTruncated(lootItems[itemIndexToLoot].bestPrice) .. ")"
                DCI_topText2:SetText("Any of these items are worth LESS than the loot:") --default message before checking item values

                if DCI_CompareByPrice == 1 or not UseAuctionPrices() then
                    highlightValue = lootItems[itemIndexToLoot].sellPrice
                elseif DCI_CompareByPrice == 2 then
                    if lootItems[itemIndexToLoot].auctionPrice ~= nil then
                        highlightValue = lootItems[itemIndexToLoot].auctionPrice
                    else
                        highlightValue = lootItems[itemIndexToLoot].sellPrice
                    end
                elseif DCI_CompareByPrice == 3 then
                    highlightValue = lootItems[itemIndexToLoot].bestPrice
                end
            end

            --loop set text until it fits in the window, shrinking as needed. 
            local shortLink = lootItems[itemIndexToLoot].itemLink or lootItems[itemIndexToLoot].itemName
            local itemCharacterLimit = string.len(lootItems[itemIndexToLoot].itemName)
            local textShrinkCounter = 0 --give up if we try too many times
            local DCIFrame_Width,_ = DCIFrame:GetSize()
            local topText_Width = 0
            repeat
                DCIPrint("Item string attempt " .. textShrinkCounter .. " - char limit " ..itemCharacterLimit .. " - new string ".. shortLink)
                local itemToLootText = ""
                if lootItems[itemIndexToLoot].quantity > 1 then
                    itemToLootText = itemToLootText .. lootItems[itemIndexToLoot].quantity .. "x"
                end                
                if shortLink then
                    itemToLootText = itemToLootText .. shortLink
                end

                DCI_topText1:SetText("Looting " .. itemToLootText .. endOfItemString)

                topText_Width,_ = DCI_topText1:GetSize()
                textShrinkCounter = textShrinkCounter + 1
                itemCharacterLimit = itemCharacterLimit - 1
                if textShrinkCounter > 25 then itemCharacterLimit = 5 end
                shortLink = string.gsub(lootItems[itemIndexToLoot].itemLink, lootItems[itemIndexToLoot].itemName, string.sub(lootItems[itemIndexToLoot].itemName, 1, itemCharacterLimit) .. "...")
            until itemCharacterLimit <= 5 or (topText_Width + 15) < DCIFrame_Width or textShrinkCounter > 25
        end
    else
        --no loot frame or quest frame. no item text or highlight
        highlightValue = nil
        DCI_topText1:SetText("") 
        DCI_topText2:SetText("") 
    end

    DCIPrint("Updating displayed items")
    --get sorted items array
    local bagItems = GetSortedBagItems()

    --show message if we have no items
    if #bagItems == 0 then
        DCI_topText1:SetText("|cffddddddNone of your items match the current filters.|r")
        DCI_topText2:SetText("|cff666666Click the info circle to change your settings.|r")
    end

    --hide items by default
    for itemNum = 1,4 do
        DCI_itemButton[itemNum]:Hide()
    end
    --loop N cheapest items
    for itemNum = 1,min(4,#bagItems) do
        --show item
        DCI_itemButton[itemNum]:Show()

        --item icon texture
        DCI_itemTexture[itemNum]:SetTexture(GetItemIcon(bagItems[itemNum].itemID))

        --show the quantity
        DCI_countText[itemNum]:SetText(bagItems[itemNum].stackCount)

        --grey out poor quality
        if bagItems[itemNum].quality == 0 then
            DCI_itemTexture[itemNum]:SetAlpha(0.5)
            DCI_countText[itemNum]:SetAlpha(0.5)
        else
            DCI_itemTexture[itemNum]:SetAlpha(1)
            DCI_countText[itemNum]:SetAlpha(1)
        end

        -- Glow texture for items that are more valuable than the loot
        DCI_glowTexture[itemNum]:Hide()
        if highlightValue then
            local highlightCount = 0
            local bagItemComparePrice = 0
            if DCI_CompareByPrice == 1 or not UseAuctionPrices() then
                bagItemComparePrice = bagItems[itemNum].vendorPriceTotal
            elseif DCI_CompareByPrice == 2 then
                if bagItems[itemNum].auctionPriceTotal ~= nil then
                    bagItemComparePrice = bagItems[itemNum].auctionPriceTotal
                else
                    bagItemComparePrice = bagItems[itemNum].vendorPriceTotal
                end
            elseif DCI_CompareByPrice == 3 then
                bagItemComparePrice = bagItems[itemNum].bestPriceTotal
            end

            if bagItemComparePrice >= highlightValue then
                highlightCount = highlightCount + 1
                DCI_glowTexture[itemNum]:Show()
            end
            if (highlightCount > 0) then
                DCI_topText2:SetText("Caution: Items in RED are worth MORE than the loot.")
            end
        end

        --display item values
        if UseAuctionPrices() and DCI_ShowBothPrices then
            --top line vendor price
            DCI_itemTextTop1[itemNum]:SetText("Vendor")
            DCI_itemTextTop2[itemNum]:SetText(GetMoneyStringTruncated(bagItems[itemNum].vendorPriceTotal))
            DCI_itemTextTop2[itemNum]:Show()

            --bottom line vendor price
            DCI_itemTextBottom1[itemNum]:SetText("Auction")
            if bagItems[itemNum].auctionPriceTotal and bagItems[itemNum].auctionPriceTotal > 0 then
                DCI_itemTextBottom2[itemNum]:SetText(GetMoneyStringTruncated(bagItems[itemNum].auctionPriceTotal))
                --grey auction price if cheaper than vendor based on global DCI_AuctionGreyUnderVendor
                if DCI_AuctionGreyUnderVendor and bagItems[itemNum].auctionPriceTotal <= bagItems[itemNum].vendorPriceTotal then
                    DCI_itemTextBottom1[itemNum]:SetTextColor(0.3,0.3,0.3)
                    DCI_itemTextBottom2[itemNum]:SetTextColor(0.3,0.3,0.3)
                else
                    DCI_itemTextBottom1[itemNum]:SetTextColor(1,0.82,0)
                    DCI_itemTextBottom2[itemNum]:SetTextColor(1,0.82,0)
                end
            else
                DCI_itemTextBottom2[itemNum]:SetText("- ")
                DCI_itemTextBottom1[itemNum]:SetTextColor(0.3,0.3,0.3)
                DCI_itemTextBottom2[itemNum]:SetTextColor(0.3,0.3,0.3)
            end
        else
            --shorten long item names
            local shortName = bagItems[itemNum].itemName
            if string.len(shortName) > 19 then
                shortName = string.sub(bagItems[itemNum].itemName, 1, 17) .. "..."
            end

            --top line item name
            local r,g,b,hex = GetItemQualityColor(bagItems[itemNum].quality)
            DCI_itemTextTop1[itemNum]:SetText("|c" .. hex .. shortName .. "|r")
            DCI_itemTextTop2[itemNum]:Hide()

            --bottom line price
            if DCI_CompareByPrice == 1 then
                --use vendor price
                DCI_itemTextBottom1[itemNum]:SetText("Vendor")
                DCI_itemTextBottom2[itemNum]:SetText(GetMoneyStringTruncated(bagItems[itemNum].vendorPriceTotal))
            elseif DCI_CompareByPrice == 2 then
                --use auction price IF its available
                if bagItems[itemNum].auctionPriceTotal then
                    DCI_itemTextBottom1[itemNum]:SetText("Auction")
                    DCI_itemTextBottom2[itemNum]:SetText(GetMoneyStringTruncated(bagItems[itemNum].auctionPriceTotal))
                else
                    DCI_itemTextBottom1[itemNum]:SetText("Vendor")
                    DCI_itemTextBottom2[itemNum]:SetText(GetMoneyStringTruncated(bagItems[itemNum].vendorPriceTotal))
                end
            elseif DCI_CompareByPrice == 3 then
                --use best price
                if bagItems[itemNum].auctionPriceTotal and bagItems[itemNum].auctionPriceTotal > bagItems[itemNum].vendorPriceTotal then
                    DCI_itemTextBottom1[itemNum]:SetText("Auction")
                    DCI_itemTextBottom2[itemNum]:SetText(GetMoneyStringTruncated(bagItems[itemNum].auctionPriceTotal))
                else
                    DCI_itemTextBottom1[itemNum]:SetText("Vendor")
                    DCI_itemTextBottom2[itemNum]:SetText(GetMoneyStringTruncated(bagItems[itemNum].vendorPriceTotal))
                end
            end
            --reset bottom color
            DCI_itemTextBottom1[itemNum]:SetTextColor(1,0.82,0)
            DCI_itemTextBottom2[itemNum]:SetTextColor(1,0.82,0)
        end

        --Set script to show tooltip on item hover
        DCI_itemButton[itemNum]:SetScript("OnEnter", function(self)
            GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
            GameTooltip:SetItemByID(bagItems[itemNum].itemID)  
            GameTooltip:Show()
        end)
        DCI_itemButton[itemNum]:SetScript("OnLeave", function()
            GameTooltip:Hide()
        end)

        --Set script to show tooltip on item hover for delete button
        DCI_deleteButton[itemNum]:SetScript("OnEnter", function(self)
            GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
            GameTooltip:SetItemByID(bagItems[itemNum].itemID)  
            GameTooltip:AddLine(" ")
            GameTooltip:AddLine("CLICK TO DELETE", 1, 0, 0)
            GameTooltip:Show()
        end)
        DCI_deleteButton[itemNum]:SetScript("OnLeave", function()
            GameTooltip:Hide()
        end)

        --click event for delete
        DCI_deleteButton[itemNum]:SetScript("OnMouseUp", function()
            ConfirmAndDeleteBagItem(bagItems[itemNum].bag, bagItems[itemNum].slot)

            if LootFrame:IsShown() then
                TakeItemsFromLootWindow(DCI_LootIndexUserClick)
                DCI_LootIndexUserClick = nil
                UpdateDCIFrame()
            elseif QuestFrame:IsShown() then
                DCIFrame:Hide()
            end
        end)

        --click event for ignore
        DCI_ignoreButton[itemNum]:SetScript("OnMouseUp", function()
            SaveIgnoredItem(bagItems[itemNum].itemID, bagItems[itemNum].itemName)

            UpdateDCIFrame()
        end)

    end

    -- Show the frame
    if showFrameAfterUpdate then
        DCIPrint("Displaying frame and resetting wait flag")
        DCIFrame:Show()
        DCI_InvErrWaitForWindow = false
    end
end

--function to create settings panel
local function CreateDCISettings()
    -- Create a configuration panel frame
    local ConfigFrame = CreateFrame("Frame", "DCI_ConfigPanel", UIParent)
    ConfigFrame.name = "Delete Cheapest Item" 
    InterfaceOptions_AddCategory(ConfigFrame)

    -- Title for the settings panel
    local title = ConfigFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
    title:SetPoint("TOPLEFT", 16, -16)
    title:SetText("Delete Cheapest Item")

    --button to reset ignore list
    local resetIgnoreButton = CreateFrame("Button","resetIgnoreButton", ConfigFrame, "UIPanelButtonTemplate") 
    resetIgnoreButton:SetText("Reset Ignored Items")
    resetIgnoreButton:SetSize(150, 32)  -- Set the size of the texture
    resetIgnoreButton:SetPoint("TOPLEFT", title, "TOPLEFT", 0, -35)
    resetIgnoreButton:SetScript("OnMouseUp", function()
        DCIPrint("Resetting ignored items list.", 0)

        DCI_IgnoredItems = {}
        UpdateDCIFrame(false)
    end)
    local previousSetting = resetIgnoreButton

    --button to show test window
    local testWindowButton = CreateFrame("Button","testWindowButton", ConfigFrame, "UIPanelButtonTemplate") 
    testWindowButton:SetText("Open Test Window")
    testWindowButton:SetSize(150, 32)  -- Set the size of the texture
    testWindowButton:SetPoint("LEFT", resetIgnoreButton, "RIGHT", 20, 0)
    testWindowButton:SetScript("OnMouseUp", function()
        if DCIFrame and DCIFrame:IsShown() then
            DCIFrame:Hide()
        else
            UpdateDCIFrame()
        end
    end)
    local previousSetting = resetIgnoreButton

    --checkbox for DCI_AllowInCombat
    local combatCheckbox = CreateFrame("CheckButton", "combatCheckbox", ConfigFrame, "UICheckButtonTemplate")
    combatCheckbox:SetPoint("TOPLEFT", previousSetting, "BOTTOMLEFT", 0, -15)
    combatCheckbox:SetChecked(DCI_AllowInCombat)
    combatCheckbox:SetScript("OnClick", function(self) DCI_AllowInCombat = self:GetChecked() UpdateDCIFrame(false) end)
    local label_combatCheckbox = ConfigFrame:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
    label_combatCheckbox:SetPoint("TOPLEFT", combatCheckbox, "TOPRIGHT", 0, -10)
    label_combatCheckbox:SetText("Allow window in combat")
    local previousSetting = combatCheckbox

    --checkbox for DCI_HideSoulbound
    local soulboundCheckbox = CreateFrame("CheckButton", "combatCheckbox", ConfigFrame, "UICheckButtonTemplate")
    soulboundCheckbox:SetPoint("TOPLEFT", previousSetting, "BOTTOMLEFT", 0, 0)
    soulboundCheckbox:SetChecked(DCI_HideSoulbound)
    soulboundCheckbox:SetScript("OnClick", function(self) DCI_HideSoulbound = self:GetChecked() UpdateDCIFrame(false) end)
    local label_soulboundCheckbox = ConfigFrame:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
    label_soulboundCheckbox:SetPoint("TOPLEFT", soulboundCheckbox, "TOPRIGHT", 0, -10)
    label_soulboundCheckbox:SetText("Hide soulbound items")
    local previousSetting = soulboundCheckbox

    --dropdown for DCI_MaxQuality
    local qualityFrame = CreateFrame("Frame", "QualityFrame", ConfigFrame, "UIDropDownMenuTemplate")
    qualityFrame:SetPoint("TOPLEFT", previousSetting, "BOTTOMLEFT", 260, -3)
    --qualityFrame:SetSize(150, 30)
    local label_maxQuality = ConfigFrame:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
    label_maxQuality:SetPoint("TOPLEFT", previousSetting, "BOTTOMLEFT", 0, -10)
    label_maxQuality:SetText("         Max item quality to display for deletion: ")
    local previousSetting = label_maxQuality

    --dropdown for DCI_ConfirmMinQuality
    local confirmQualityFrame = CreateFrame("Frame", "confirmQualityFrame", ConfigFrame, "UIDropDownMenuTemplate")
    confirmQualityFrame:SetPoint("TOPLEFT", previousSetting, "BOTTOMLEFT", 260, -13)
    --qualityFrame:SetSize(150, 30)
    local label_confirmQuality = ConfigFrame:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
    label_confirmQuality:SetPoint("TOPLEFT", previousSetting, "BOTTOMLEFT", 0, -20)
    label_confirmQuality:SetText("         Min item quality to confirm delete: ")
    local previousSetting = label_confirmQuality

    --checkbox for debug messages
    local debugCheckbox = CreateFrame("CheckButton", "debugCheckbox", ConfigFrame, "UICheckButtonTemplate")
    debugCheckbox:SetPoint("TOPLEFT", previousSetting, "BOTTOMLEFT", 0, -10)
    debugCheckbox:SetChecked(DCI_DebugOutput ~= nil)
    debugCheckbox:SetScript("OnClick", function(self) DCI_DebugOutput = self:GetChecked() and 1 or nil UpdateDCIFrame(false) end)
    local label_debugCheckbox = ConfigFrame:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
    label_debugCheckbox:SetPoint("TOPLEFT", debugCheckbox, "TOPRIGHT", 0, -10)
    label_debugCheckbox:SetText("Show debug output")
    local previousSetting = debugCheckbox

    --checkbox for DCI_UseAuctionPrices
    local auctionCheckbox = CreateFrame("CheckButton", "auctionCheckbox", ConfigFrame, "UICheckButtonTemplate")
    auctionCheckbox:SetPoint("TOPLEFT", previousSetting, "BOTTOMLEFT", 0, -50)
    auctionCheckbox:SetChecked(DCI_UseAuctionPrices)
    local label_auctionCheckbox = ConfigFrame:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
    label_auctionCheckbox:SetPoint("TOPLEFT", auctionCheckbox, "TOPRIGHT", 0, -10)
    label_auctionCheckbox:SetText("Use auction prices")
    local label_auctionCheckbox2 = ConfigFrame:CreateFontString(nil, "ARTWORK", "GameFontDisableSmall")
    label_auctionCheckbox2:SetPoint("TOPLEFT", label_auctionCheckbox, "TOPRIGHT", 10, -1)
    local AuctionSettingsList = {}
    local previousSetting = auctionCheckbox

    -- dropdown for DCI_CompareByPrice
    local compareByFrame = CreateFrame("Frame", "compareByFrame", ConfigFrame, "UIDropDownMenuTemplate")
    compareByFrame:SetPoint("TOPLEFT", previousSetting, "BOTTOMLEFT", 260, -3)
    --compareByFrame:SetSize(250, 30)
    local label_compareBy = ConfigFrame:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
    label_compareBy:SetPoint("TOPLEFT", previousSetting, "BOTTOMLEFT", 0, -10)
    label_compareBy:SetText("         Sort and Compare item values based on: ")
    local previousSetting = label_compareBy
    table.insert(AuctionSettingsList, label_compareBy)
    table.insert(AuctionSettingsList, compareByFrame)

    -- dropdown for DCI_ShowBothPrices
    local showBothFrame = CreateFrame("Frame", "showBothFrame", ConfigFrame, "UIDropDownMenuTemplate")
    showBothFrame:SetPoint("TOPLEFT", previousSetting, "BOTTOMLEFT", 260, -13)
    --showBothFrame:SetSize(200, 30)
    local label_showBoth = ConfigFrame:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
    label_showBoth:SetPoint("TOPLEFT", previousSetting, "BOTTOMLEFT", 0, -20)
    label_showBoth:SetText("         Lines to show: ")
    local previousSetting = label_showBoth
    table.insert(AuctionSettingsList, label_showBoth)
    table.insert(AuctionSettingsList, showBothFrame)

    --checkbox for DCI_AuctionGreyUnderVendor
    local auctionGreyCheckbox = CreateFrame("CheckButton", "auctionGreyCheckbox", ConfigFrame, "UICheckButtonTemplate")
    auctionGreyCheckbox:SetPoint("TOPLEFT", previousSetting, "BOTTOMLEFT", 0, -10)
    auctionGreyCheckbox:SetChecked(DCI_AuctionGreyUnderVendor)
    auctionGreyCheckbox:SetScript("OnClick", function(self) DCI_AuctionGreyUnderVendor = self:GetChecked() UpdateDCIFrame(false) end)
    local label_auctionGreyCheckbox = ConfigFrame:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
    label_auctionGreyCheckbox:SetPoint("TOPLEFT", auctionGreyCheckbox, "TOPRIGHT", 0, -10)
    label_auctionGreyCheckbox:SetText("Grey out auction price if it's less than the vendor price (when both are shown)")
    table.insert(AuctionSettingsList, auctionGreyCheckbox)
    table.insert(AuctionSettingsList, label_auctionGreyCheckbox)
    local previousSetting = auctionGreyCheckbox

    --checkbox for DCI_SubtractAuctionCut
    local auctionCutCheckbox = CreateFrame("CheckButton", "auctionCutCheckbox", ConfigFrame, "UICheckButtonTemplate")
    auctionCutCheckbox:SetPoint("TOPLEFT", previousSetting, "BOTTOMLEFT", 0, 0)
    auctionCutCheckbox:SetChecked(DCI_SubtractAuctionCut)
    auctionCutCheckbox:SetScript("OnClick", function(self) DCI_SubtractAuctionCut = self:GetChecked() UpdateDCIFrame(false) end)
    local label_auctionCutCheckbox = ConfigFrame:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
    label_auctionCutCheckbox:SetPoint("TOPLEFT", auctionCutCheckbox, "TOPRIGHT", 0, -10)
    label_auctionCutCheckbox:SetText("Subtract 5% cut from auction price")
    table.insert(AuctionSettingsList, auctionCutCheckbox)
    table.insert(AuctionSettingsList, label_auctionCutCheckbox)
    local previousSetting = auctionCutCheckbox

    --checkbox for DCI_AuctionPriceHidePoorQuality
    local auctionPoorCheckbox = CreateFrame("CheckButton", "auctionCutCheckbox", ConfigFrame, "UICheckButtonTemplate")
    auctionPoorCheckbox:SetPoint("TOPLEFT", previousSetting, "BOTTOMLEFT", 0, 0)
    auctionPoorCheckbox:SetChecked(DCI_AuctionPriceHidePoorQuality)
    auctionPoorCheckbox:SetScript("OnClick", function(self) DCI_AuctionPriceHidePoorQuality = self:GetChecked() UpdateDCIFrame(false) end)
    local label_auctionPoorCheckbox = ConfigFrame:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
    label_auctionPoorCheckbox:SetPoint("TOPLEFT", auctionPoorCheckbox, "TOPRIGHT", 0, -10)
    label_auctionPoorCheckbox:SetText("Ignore auction price for grey quality items")
    table.insert(AuctionSettingsList, auctionPoorCheckbox)
    table.insert(AuctionSettingsList, label_auctionPoorCheckbox)
    local previousSetting = auctionPoorCheckbox

    --dropdown settings
    local qualityOptions = {"Common", "Uncommon", "Rare", "Epic", "Legendary"} qualityOptions[0] = "Poor"
    local compareByOptions = {"Vendor Price Only", "Auction Price (if known, otherwise Vendor)", "Highest Price (Vendor or Auction)"}
    local showBothOptions = {"Item Name + Selected Price", "Both Prices (Vendor + Auction)"}
    local showBothValues = {false, true}

    --function to update auction settings enable/disable state
    local function UpdateAuctionSettings()
        --if no API available at all, disable the auction setting itself
        if not UseAuctionPrices(true) then
            auctionCheckbox:SetChecked(false)
            auctionCheckbox:Disable()
            label_auctionCheckbox:SetTextColor(0.5, 0.5, 0.5)
            label_auctionCheckbox2:SetText("(No auction price addon data found)")
        else
            auctionCheckbox:Enable()
            label_auctionCheckbox:SetTextColor(1, 1, 1)
            if DCI_AuctionSource then
                label_auctionCheckbox2:SetText("(sourced from " .. DCI_AuctionSource .. ")")
            else
                label_auctionCheckbox2:SetText("")
            end
        end

        --the rest are based on whether the user setting for auctions are enabled
        if UseAuctionPrices() then
            --enable checkboxes
            for _, AuctionSettingToChange in ipairs(AuctionSettingsList) do
                if AuctionSettingToChange.Enable then AuctionSettingToChange:Enable() end
                if AuctionSettingToChange.SetTextColor then AuctionSettingToChange:SetTextColor(1, 1, 1) end
                AuctionSettingToChange:Show()
            end
            UIDropDownMenu_SetText(compareByFrame, compareByOptions[DCI_CompareByPrice])
            UIDropDownMenu_EnableDropDown(compareByFrame)

            --if not DCI_ShowBothPrices, then disable the Grey Auction Price setting
            if DCI_ShowBothPrices then
                auctionGreyCheckbox:Enable()
                label_auctionGreyCheckbox:SetTextColor(1, 1, 1)
            else
                auctionGreyCheckbox:Disable()
                label_auctionGreyCheckbox:SetTextColor(0.5, 0.5, 0.5)
            end

        else
            --disable checkboxes
            for _, AuctionSettingToChange in ipairs(AuctionSettingsList) do
                if AuctionSettingToChange.Disable then AuctionSettingToChange:Disable() end
                if AuctionSettingToChange.SetTextColor then AuctionSettingToChange:SetTextColor(0.5, 0.5, 0.5) end
                AuctionSettingToChange:Hide()
            end
            UIDropDownMenu_SetText(compareByFrame, "|cff7f7f7f"..compareByOptions[DCI_CompareByPrice].."|r")
            UIDropDownMenu_DisableDropDown(compareByFrame)
        end
    end
    ConfigFrame:SetScript("OnShow", function(self) UpdateAuctionSettings() end)

    --handle quality dropdown menu
    local function InitializeDropDownQuality(self, level)
        local info = UIDropDownMenu_CreateInfo()
        for qualityID = 0, #qualityOptions do
            info.text = qualityOptions[qualityID]
            info.value = qualityOptions[qualityID]
            local r,g,b,hex = GetItemQualityColor(qualityID)
            info.colorCode = "|c" .. hex
            info.func = function()
                DCI_MaxQuality = qualityID
                local r,g,b,hex = GetItemQualityColor(DCI_MaxQuality)
                UIDropDownMenu_SetText(qualityFrame, "|c" .. hex .. qualityOptions[DCI_MaxQuality] .. "|r")

                --also update the min confirm quality if its above the new max quality
                r,g,b,hex = 1,0,0,"ffff0000"
                local confirmQuality = "Never"
                if DCI_ConfirmMinQuality and DCI_ConfirmMinQuality <= DCI_MaxQuality then
                    r,g,b,hex = GetItemQualityColor(DCI_ConfirmMinQuality)
                    confirmQuality = qualityOptions[DCI_ConfirmMinQuality]
                end
                UIDropDownMenu_SetText(confirmQualityFrame, "|c" .. hex .. confirmQuality .. "|r")

                UpdateDCIFrame(false)
            end

            UIDropDownMenu_AddButton(info, level)
        end
    end
    UIDropDownMenu_Initialize(qualityFrame, InitializeDropDownQuality)
    UIDropDownMenu_SetWidth(qualityFrame, 120)
    UIDropDownMenu_SetButtonWidth(qualityFrame, 133)
    local r,g,b,hex = GetItemQualityColor(DCI_MaxQuality)
    UIDropDownMenu_SetText(qualityFrame, "|c" .. hex .. qualityOptions[DCI_MaxQuality] .. "|r")

    --handle confirmation quality dropdown menu
    local function InitializeDropDownConfirm(self, level)
        local info = UIDropDownMenu_CreateInfo()
        for qualityID = 0, DCI_MaxQuality do
            info.text = qualityOptions[qualityID]
            info.value = qualityID
            local r,g,b,hex = GetItemQualityColor(qualityID)
            info.colorCode = "|c" .. hex
            info.func = function()
                DCI_ConfirmMinQuality = qualityID
                local r,g,b,hex = 1,0,0,"ffff0000"
                local confirmQuality = "Never"
                if DCI_ConfirmMinQuality ~= nil then
                    r,g,b,hex = GetItemQualityColor(DCI_ConfirmMinQuality)
                    confirmQuality = qualityOptions[DCI_ConfirmMinQuality]
                end
                UIDropDownMenu_SetText(confirmQualityFrame, "|c" .. hex .. confirmQuality .. "|r")
                UpdateDCIFrame(false)
            end

            UIDropDownMenu_AddButton(info, level)
        end

        --add extra option for never
        info.text = "Never"
        info.value = nil
        info.colorCode = "|cffff0000"
        info.func = function()
            DCI_ConfirmMinQuality = nil
            UIDropDownMenu_SetText(confirmQualityFrame, "|cffff0000Never|r")
            UpdateDCIFrame(false)
        end
        UIDropDownMenu_AddButton(info, level)
    end
    UIDropDownMenu_Initialize(confirmQualityFrame, InitializeDropDownConfirm)
    UIDropDownMenu_SetWidth(confirmQualityFrame, 120)
    UIDropDownMenu_SetButtonWidth(confirmQualityFrame, 133)
    local r,g,b,hex = 1,0,0,"ffff0000"
    local confirmQuality = "Never"
    if DCI_ConfirmMinQuality and DCI_ConfirmMinQuality <= DCI_MaxQuality then
        r,g,b,hex = GetItemQualityColor(DCI_ConfirmMinQuality)
        confirmQuality = qualityOptions[DCI_ConfirmMinQuality]
    end
    UIDropDownMenu_SetText(confirmQualityFrame, "|c" .. hex .. confirmQuality .. "|r")

    -- handle dropdown menu for comparison price
    local function InitializeDropDownCompareBy(self, level)
        local info = UIDropDownMenu_CreateInfo()
        for i = 1,#compareByOptions do
            info.text = compareByOptions[i]
            info.value = i

            info.func = function()
                DCI_CompareByPrice = i
                UIDropDownMenu_SetText(compareByFrame, compareByOptions[i])
                UpdateDCIFrame(false)
            end

            UIDropDownMenu_AddButton(info, level)
        end
    end
    UIDropDownMenu_Initialize(compareByFrame, InitializeDropDownCompareBy)
    UIDropDownMenu_SetWidth(compareByFrame, 250)
    UIDropDownMenu_SetButtonWidth(compareByFrame, 263)
    UIDropDownMenu_SetText(compareByFrame, compareByOptions[DCI_CompareByPrice])

   -- handle dropdown menu for DCI_ShowBothPrices
    local function InitializeDropDownShowBoth(self, level)
        local info = UIDropDownMenu_CreateInfo()
        for i = 1,#showBothOptions do
            info.text = showBothOptions[i]
            info.value = showBothValues[i]

            info.func = function()
                DCI_ShowBothPrices = showBothValues[i]
                UIDropDownMenu_SetText(showBothFrame, showBothOptions[i])
                UpdateDCIFrame(false)
                UpdateAuctionSettings()
            end

            UIDropDownMenu_AddButton(info, level)
        end
    end
    UIDropDownMenu_Initialize(showBothFrame, InitializeDropDownShowBoth)
    UIDropDownMenu_SetWidth(showBothFrame, 250)
    UIDropDownMenu_SetButtonWidth(showBothFrame, 263)
    UIDropDownMenu_SetText(showBothFrame, showBothOptions[DCI_ShowBothPrices and 2 or 1])

    --handle auction price checkbox click
    auctionCheckbox:SetScript("OnClick", function(self)
        DCI_UseAuctionPrices = self:GetChecked()
        UpdateDCIFrame(false)
        UpdateAuctionSettings()
    end)
end

--register events
local DCIEventFrame = CreateFrame("Frame")
DCIEventFrame:RegisterEvent("ADDON_LOADED")
DCIEventFrame:RegisterEvent("UI_ERROR_MESSAGE")
DCIEventFrame:RegisterEvent("LOOT_CLOSED")
DCIEventFrame:RegisterEvent("LOOT_OPENED")
DCIEventFrame:RegisterEvent("BAG_UPDATE")
DCIEventFrame:RegisterEvent("PLAYER_REGEN_DISABLED")
DCIEventFrame:RegisterEvent("PLAYER_REGEN_ENABLED")
DCIEventFrame:RegisterEvent("QUEST_COMPLETE")
DCIEventFrame:SetScript("OnEvent", function(_, event, ...)
    local arg1,arg2 = ...

    --initial addon load
    if event == "ADDON_LOADED" and arg1 == "DeleteCheapestItem" then
        DCIPrint("Initializing")
        --initialize saved variables
        if DCI_IgnoredItems == nil then DCI_IgnoredItems = {} end
        if DCI_UseAuctionPrices == nil then DCI_UseAuctionPrices = (Auctionator ~= nil) end
        if DCI_SubtractAuctionCut == nil then DCI_SubtractAuctionCut = true end
        if DCI_AuctionPriceHidePoorQuality == nil then DCI_AuctionPriceHidePoorQuality = true end
        if DCI_AuctionGreyUnderVendor == nil then DCI_AuctionGreyUnderVendor = true end
        if DCI_HideSoulbound == nil then DCI_HideSoulbound = true end
        if DCI_AllowInCombat == nil then DCI_AllowInCombat = false end
        if DCI_MaxQuality == nil then DCI_MaxQuality = 2 end
        if DCI_CompareByPrice == nil then DCI_CompareByPrice = 3 end --{"Vendor Price", "Auction Price", "Highest Price"}
        if DCI_ShowBothPrices == nil then DCI_ShowBothPrices = false end
        if DCI_ConfirmMinQuality == nil then DCI_ConfirmMinQuality = nil end --default to nil which represents never
        if DCI_DebugOutput == nil then DCI_DebugOutput = nil end --default to nil which represents no debug

        --globals
        if DCI_WindowMoved == nil then DCI_WindowMoved = false end

        -- Register slash command
        SLASH_DCI1 = "/dci"
        SlashCmdList["DCI"] = function() UpdateDCIFrame() end

        --create settings screen
        CreateDCISettings()
    end

    --update frame if bag updates while it is open
    if event == "BAG_UPDATE" and DCIFrame and DCIFrame:IsShown() then
        DCIPrint("Updating window due to bag update")
        UpdateDCIFrame()
    end

    --react to full inv error
    if event == "UI_ERROR_MESSAGE" and arg2 == ERR_INV_FULL then
        DCIPrint("Inventory error")
        --if loot or dci frame exists then update
        if (LootFrame and LootFrame:IsShown()) or (DCIFrame and DCIFrame:IsShown()) then
            DCIPrint("Updating window since LootFrame (or DCIFrame) is shown")
            UpdateDCIFrame()
        
        --no loot frame yet, set flag to wait for window
        else
            DCIPrint("No loot frame, set flag to wait")
            DCI_InvErrWaitForWindow = true
        end
    end

    --when loot window shown, hook onto loot item click to save clicked index
    if event == "LOOT_OPENED" then
        for i = 1,GetNumLootItems() do
            if _G['LootButton'..i] and not _G['LootButton'..i].DCIFunctionAltered then
                DCIPrint("Setting hook on loot item click")
                _G['LootButton'..i]:HookScript("OnClick", function()
                    DCI_LootIndexUserClick = i
                    DCIPrint("LootButton click, force index but dont update window " .. DCI_LootIndexUserClick)
                    --UpdateDCIFrame(false)
                end)
                _G['LootButton'..i].DCIFunctionAltered = true
            end
        end
    end

    --show with loot window, if we already saw the inv error, or if the window is open already
    if event == "LOOT_OPENED" and (DCI_InvErrWaitForWindow or (DCIFrame and DCIFrame:IsShown())) then
        DCIPrint("Loot window opened")
        --if we're not in combat (or dont care)
        if DCI_AllowInCombat == true or not UnitAffectingCombat("player") then
            DCIPrint("Not in combat or don't care")
            DCIPrint("Update window and reset wait flag")
            UpdateDCIFrame()
            DCI_InvErrWaitForWindow = false
        end        
    end

    --show with quest complete, if there are more quest rewards than bag slots
    --(this isn't smart enough to know if any quest rewards could add to a stack in your bags, we just assume not)
    local QuestRewardRequiredSlots = ((GetNumQuestChoices() > 0) and 1 or 0) + GetNumQuestRewards()
    if event == "QUEST_COMPLETE" and QuestFrame:IsShown() and QuestRewardRequiredSlots > CountFreeBagSlots() then
        DCIPrint("Quest complete, and there are more rewards than bag slots - show window")
        UpdateDCIFrame()

        --hook onto reward selections to update comparison item index
        for i = 1,GetNumQuestChoices() do
            if _G['QuestInfoRewardsFrameQuestInfoItem'..i] then
                _G['QuestInfoRewardsFrameQuestInfoItem'..i]:HookScript("OnClick", function() DCI_QuestRewardSelected = i UpdateDCIFrame(false) end)
            end
        end
    end

    --hide with loot window closing
    if event == "LOOT_CLOSED" then
        if DCIFrame and DCIFrame:IsShown() then
            DCIPrint("Closing with loot window. reset wait flag")
            DCIFrame:Hide()
            --DCI_InvErrWaitForWindow = true
        end
    end

    --hide if we enter combat, based on global DCI_AllowInCombat
    if DCI_AllowInCombat == false and event == "PLAYER_REGEN_DISABLED" then
        if DCIFrame and DCIFrame:IsShown() then
            DCIPrint("Hiding due to combat, setting wait flag*")
            DCIFrame:Hide()
            --DCI_InvErrWaitForWindow = true
        end
    end

    --show if we leave combat, loot window is already open, and DCI_InvErrWaitForWindow is true
    if DCI_AllowInCombat == false and event == "PLAYER_REGEN_ENABLED" and LootFrame:IsShown() and DCI_InvErrWaitForWindow then
        DCIPrint("Combat ended and wait flag is on, show window and reset flag")
        UpdateDCIFrame()
        DCI_InvErrWaitForWindow = false
    end

    DCIPrint(event .. " - " .. dump(...))
end)